/*
 * Utility functions
 */
/*
 *  This is part of ypp-sc-tools, a set of third-party tools for assisting
 *  players of Yohoho Puzzle Pirates.
 * 
 *  Copyright (C) 2009 Ian Jackson <ijackson@chiark.greenend.org.uk>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  Yohoho and Puzzle Pirates are probably trademarks of Three Rings and
 *  are used without permission.  This program is not endorsed or
 *  sponsored by Three Rings.
 */

#include "common.h"

void *mmalloc(size_t sz) {
  void *r;
  if (!sz) return 0;
  sysassert( r= malloc(sz) );
  return r;
}
void *mrealloc(void *p, size_t sz) {
  assert(sz);
  void *r;
  sysassert( r= realloc(p,sz) );
  return r;
}


FILE *dbfile;
static const char *path;

int dbfile_open(const char *tpath) {
  assert(!dbfile);
  path= tpath;
  dbfile= fopen(path,"r");
  if (dbfile) return 1;
  sysassert(errno==ENOENT);
  return 0;
}  

#define dbassertgl(x) ((x) ? (void)0 : dbfile_assertfail(file,line,#x))

void dbfile_getsline(char *lbuf, size_t lbufsz, const char *file, int line) {
  errno=0;
  char *s= fgets(lbuf,lbufsz,dbfile);
  sysassert(!ferror(dbfile));
  dbassertgl(!feof(dbfile));
  assert(s);
  int l= strlen(lbuf);
  dbassertgl(l>0);  dbassertgl(lbuf[--l]=='\n');
  lbuf[l]= 0;
}

void dbfile_close(void) {
  if (!dbfile) return;
  sysassert(!ferror(dbfile));
  sysassert(!fclose(dbfile));
  dbfile= 0;
}

int dbfile_vscanf(const char *fmt, va_list al) {
  int r= vfscanf(dbfile,fmt,al);
  sysassert(!ferror(dbfile));
  return r;
}

int dbfile_scanf(const char *fmt, ...) {
  va_list al;
  va_start(al,fmt);
  int r= dbfile_vscanf(fmt,al);
  va_end(al);
  return r;
}

void dbfile_assertfail(const char *file, int line, const char *m) {
  if (dbfile)
    fatal("Error in dictionary file %s at byte %ld:\n"
	  " Requirement not met at %s:%d:\n"
	  " %s",
	  path,(long)ftell(dbfile), file,line, m);
  else
    fatal("Semantic error in dictionaries:\n"
	  " Requirement not met at %s:%d:\n"
	  " %s",
	  file,line, m);
}
