/*
 * ocr.c forms a mostly-self-contained bit
 * so we put its declarations in this separate file
 */
/*
 *  This is part of ypp-sc-tools, a set of third-party tools for assisting
 *  players of Yohoho Puzzle Pirates.
 * 
 *  Copyright (C) 2009 Ian Jackson <ijackson@chiark.greenend.org.uk>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  Yohoho and Puzzle Pirates are probably trademarks of Three Rings and
 *  are used without permission.  This program is not endorsed or
 *  sponsored by Three Rings.
 */

#ifndef OCR_H
#define OCR_H

#include "common.h"


typedef uint32_t Pixcol;
#define PSPIXCOL(priscan) priscan##32

typedef struct {
  const char *s; /* valid until next call to ocr() */
  int l,r; /* column numbers */
  int match; /* match context index */
  unsigned ctxmap; /* possible match contexts */
} OcrResultGlyph;


typedef const struct OcrCellTypeInfo *OcrCellType;
extern const struct OcrCellTypeInfo ocr_celltype_text;
extern const struct OcrCellTypeInfo ocr_celltype_number;
const char *ocr_celltype_name(OcrCellType ct);


typedef struct OcrReader OcrReader;
OcrReader *ocr_init(int h);
void ocr_showcharsets(void);

OcrResultGlyph *ocr(OcrReader *rd, OcrCellType, int w, Pixcol cols[]);
  /* return value is array terminated by {0,-1,-1}
   * array is valid until next call to ocr()
   */


extern const char *o_resolver;

FILE *resolve_start(void);
void resolve_finish(void);


#endif /*OCR_H*/
